// $Id: CoreApplication.cpp,v 1.6 2014/02/28 14:47:31 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CoreApplication.h"
#include "CoreApplication.moc"

#include "Client.h"
#include "Debug.h"

#include <QCoreApplication>

//___________________________________________
CoreApplication::CoreApplication( CommandLineArguments arguments ):
    BaseCoreApplication( 0, arguments ),
    Counter( "CoreApplication" )
{}

//___________________________________________
CoreApplication::~CoreApplication( void )
{}

//___________________________________________
bool CoreApplication::initApplicationManager( void )
{

    Debug::Throw( "CoreApplication::initApplicationManager.\n" );

    // make sure application manager does not already exist
    if( _hasApplicationManager() ) return true;

    // parse arguments and forbid --no-server options
    CommandLineParser parser( commandLineParser( _arguments() ) );
    if( parser.hasFlag( "--no-server" ) )
    {

        Debug::Throw(0) << "serverlog - by definition the application cannot work in 'no-server' mode." << endl;
        Debug::Throw(0) << "Aborting." << endl;
        return false;

    } else if( BaseCoreApplication::initApplicationManager() ) {

        connect( &_applicationManager().client(), SIGNAL(commandAvailable(Server::ServerCommand)), SLOT(_append(Server::ServerCommand)) );
        return _applicationManager().client().socket().waitForConnected();

    } else return false;

}

//____________________________________________
CommandLineParser CoreApplication::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "CoreApplication::commandLineParser.\n" );
    CommandLineParser out( BaseCoreApplication::commandLineParser() );

    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerOption( "--send", tr( "string" ), tr( "send command to running application") );
    out.registerFlag( "--list", tr( "list all registered applications" ) );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;
}

//____________________________________________
void CoreApplication::usage( void ) const
{
    _usage( "serverlog" );
    commandLineParser().usage();
    return;
}

//__________________________________________________________________
void CoreApplication::sendServerCommand( Server::ServerCommand command )
{
    Debug::Throw( "CoreApplication::sendServerCommand.\n" );
    if( _hasApplicationManager() )
    {
        BaseCoreApplication::sendServerCommand( command );
        _applicationManager().client().socket().waitForBytesWritten();
    }
}

//__________________________________________________________________
void CoreApplication::requestIdentify( void )
{
    BaseCoreApplication::sendServerCommand( Server::ServerCommand::Identify );
    _applicationManager().client().socket().waitForReadyRead( 100 );
}

//___________________________________________
void CoreApplication::_append( Server::ServerCommand command )
{

    switch( command.command() )
    {
        case Server::ServerCommand::Identify:
        Debug::Throw(0) << "  " << command.id().processId() << " - " << command.id().name() << endl;
        break;

        case Server::ServerCommand::IdentifyServer:
        Debug::Throw(0) << "  " << command.id().processId() << " - " << command.id().name() << " (server)" << endl;
        break;

        default:
        break;
    }

    return;

}

//________________________________________________
bool CoreApplication::_processCommand( Server::ServerCommand command )
{
    Debug::Throw() << "CoreApplication::_processCommand - " << command.commandName() << endl;
    return BaseCoreApplication::_processCommand( command );
}
