#ifndef Application_h
#define Application_h

// $Id: CoreApplication.h,v 1.5 2014/02/28 14:47:31 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseCoreApplication.h"
#include "CommandLineArguments.h"
#include "Config.h"
#include "Counter.h"

//! display application manager server messages
class CoreApplication: public BaseCoreApplication, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    CoreApplication( CommandLineArguments );

    //! destructor
    virtual ~CoreApplication( void );

    //! initialize application manager
    bool initApplicationManager( void );

    //!@name application information
    //@{

    //! command line parser
    CommandLineParser commandLineParser( CommandLineArguments = CommandLineArguments(), bool ignoreWarnings = true ) const;

    //! usage
    void usage( void ) const;

    //! application name
    virtual QString applicationName( void ) const
    { return "ServerCommand"; }

    // application version
    virtual QString applicationVersion( void ) const
    { return VERSION; }

    // build time stamp
    virtual QString buildTimeStamp( void ) const
    { return BUILD_TIMESTAMP; }

    //@}

    public Q_SLOTS:

    //! server command
    virtual void sendServerCommand( Server::ServerCommand );

    //! server command
    virtual void requestIdentify( void );

    protected Q_SLOTS:

    //! append message to the log
    void _append( Server::ServerCommand );

    //! process request from application manager
    virtual bool _processCommand( Server::ServerCommand );

};

#endif
