#ifndef ServerCommandModel_h
#define ServerCommandModel_h

// $Id: ServerCommandModel.h,v 1.20 2014/02/18 15:15:56 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "ServerCommand.h"
#include "TreeModel.h"

#include <QColor>
#include <QIcon>
#include <QHash>

//! ServerCommand model. Stores job information for display in lists
class ServerCommandModel : public TreeModel<Server::ServerCommand>, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! column type enumeration
    enum ColumnTypes
    {
        Name,
        User,
        Display,
        Command,
        Pid,
        Time,
        nColumns
    };

    //! constructor
    ServerCommandModel( QObject* = 0 );

    //! destructor
    virtual ~ServerCommandModel( void )
    {}

    //!@name methods reimplemented from base class
    //@{

    //! return data
    virtual QVariant data( const QModelIndex&, int ) const;

    //! header data
    virtual QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole ) const;

    //! number of columns for a given index
    virtual int columnCount( const QModelIndex& = QModelIndex() ) const
    { return nColumns; }

    //@}

    //! remove
    virtual void remove( ServerCommandModel::List );

    //! clear
    virtual void clear( void );


    protected:

    //! get icon for pid
    QIcon _icon( qint64 ) const;

    //! sorting
    virtual void _sort( int column, Qt::SortOrder order = Qt::AscendingOrder )
    { _root().sort( SortFTor( (ColumnTypes) column, order ) ); }

    private Q_SLOTS:

    //! configuration
    void _updateConfiguration(void);

    private:

    //! used to sort IconCaches
    class SortFTor: public ItemModel::SortFTor
    {

        public:

        //! constructor
        SortFTor( const int& type, Qt::SortOrder order = Qt::AscendingOrder ):
            ItemModel::SortFTor( type, order )
        {}

        //! prediction
        bool operator() ( const Item& first, const Item& second ) const
        { return (*this)( first.get(), second.get() ); }

        //! prediction
        bool operator() ( Server::ServerCommand, Server::ServerCommand ) const;

    };

    //! column titles
    static const QString columnTitles_[ nColumns ];

    //! colors
    QColor identifyColor_;

    //! colors
    QColor serverColor_;

    //! true if icons must be shown
    bool showIcons_;

    //! icon size
    QSize iconSize_;

    //! empty icon
    QIcon emptyIcon_;

    //! map icon and pid
    typedef QHash<qint64, QIcon> IconMap;

    //! map icon and pid
    IconMap icons_;

};

#endif
